@@ -138,8 +138,11 @@ public:
 		instances->shutdown();
 	}
 
-	static void forceClean(IProvider* p, const char* secDbName)
+	static void forceClean(IProvider* p, CachedSecurityDatabase::Instance& instance)
 	{
+		Firebird::PathName secDbName(instance->secureDbName);
+
+		instance.reset();
 		cleanup();
 
 		ClumpletWriter dpb(ClumpletReader::dpbList, MAX_DPB_SIZE);
@@ -150,13 +153,14 @@ public:
 		dpb.insertString(isc_dpb_config, ParsedList::getNonLoopbackProviders(secDbName));
 
 		Jrd::FbLocalStatus status;
-		RefPtr<IAttachment> att(REF_NO_INCR, p->attachDatabase(&status, secDbName, dpb.getBufferLength(), dpb.getBuffer()));
+		RefPtr<IAttachment> att(REF_NO_INCR,
+			p->attachDatabase(&status, secDbName.c_str(), dpb.getBufferLength(), dpb.getBuffer()));
 		check(&status);
 
 		HANDSHAKE_DEBUG(fprintf(stderr, "Srv SRP: gfix-like attach to sec db %s\n", secDbName));
 	}
 
-	SecurityDatabase(const char* secDbName, ICryptKeyCallback* cryptCallback)
+	SecurityDatabase(CachedSecurityDatabase::Instance& instance, ICryptKeyCallback* cryptCallback)
 		: att(NULL), tra(NULL), stmt(NULL)
 	{
 		Jrd::FbLocalStatus status;
@@ -173,10 +177,10 @@ public:
 			ClumpletWriter dpb(ClumpletReader::dpbList, MAX_DPB_SIZE);
 			dpb.insertByte(isc_dpb_sec_attach, TRUE);
 			dpb.insertString(isc_dpb_user_name, SYSDBA_USER_NAME, fb_strlen(SYSDBA_USER_NAME));
-			dpb.insertString(isc_dpb_config, ParsedList::getNonLoopbackProviders(secDbName));
-			att = p->attachDatabase(&status, secDbName, dpb.getBufferLength(), dpb.getBuffer());
+			dpb.insertString(isc_dpb_config, ParsedList::getNonLoopbackProviders(instance->secureDbName));
+			att = p->attachDatabase(&status, instance->secureDbName, dpb.getBufferLength(), dpb.getBuffer());
 			check(&status);
-			HANDSHAKE_DEBUG(fprintf(stderr, "Srv SRP: attached sec db %s\n", secDbName));
+			HANDSHAKE_DEBUG(fprintf(stderr, "Srv SRP: attached sec db %s\n", instance->secureDbName));
 
 			const UCHAR tpb[] =
 			{
@@ -198,20 +202,20 @@ public:
 				class Cleanup : public SrvCleanup
 				{
 				public:
-					Cleanup(IProvider* p_p, const char* p_secDbName)
-						: p(p_p), secDbName(p_secDbName)
+					Cleanup(IProvider* p_p, CachedSecurityDatabase::Instance& p_instance)
+						: p(p_p), instance(p_instance)
 					{ }
 
 					void cleanup()
 					{
-						forceClean(p, secDbName);
+						forceClean(p, instance);
 					}
 
 				private:
 					IProvider* p;
-					const char* secDbName;
+					CachedSecurityDatabase::Instance& instance;
 				};
-				Cleanup c(p, secDbName);
+				Cleanup c(p, instance);
 
 				checkStatusVectorForMissingTable(status->getErrors(), &c);
 				status_exception::raise(&status);
@@ -284,6 +288,10 @@ int SrpServer::authenticate(CheckStatusWrapper* status, IServerBlock* sb, IWrite
 
 			account = sb->getLogin();
 
+			const size_t len = account.length();
+			if (len > SZ_LOGIN)
+				status_exception::raise(Arg::Gds(isc_long_login) << Arg::Num(len) << Arg::Num(SZ_LOGIN));
+
 			unsigned int length;
 			const unsigned char* val = sb->getData(&length);
 			clientPubKey.assign(val, length);
@@ -310,7 +318,7 @@ int SrpServer::authenticate(CheckStatusWrapper* status, IServerBlock* sb, IWrite
 
 				// Create SecurityDatabase if needed
 				if (!instance->secDb)
-					instance->secDb = FB_NEW SecurityDatabase(instance->secureDbName, cryptCallback);
+					instance->secDb = FB_NEW SecurityDatabase(instance, cryptCallback);
 
 				// Lookup
 				instance->secDb->lookup(messages.param.getData(), messages.data.getData());
