@@ -423,7 +423,7 @@ void registerLegacyServer(IPluginManager* iPlugin)
 
 #ifdef PLUG_MODULE
 
-extern "C" void FB_EXPORTED FB_PLUGIN_ENTRY_POINT(IMaster* master)
+extern "C" FB_DLL_EXPORT void FB_PLUGIN_ENTRY_POINT(IMaster* master)
 {
 	CachedMasterInterface::set(master);
 
