@@ -30,6 +30,9 @@
 #define CLASSES_FPE_CONTROL_H
 
 #include <math.h>
+#ifdef HAVE_CMATH
+#include <cmath>
+#endif
 #if defined(WIN_NT)
 #include <float.h>
 #else
@@ -213,8 +216,16 @@ private:
 
 };
 
-} //namespace Firebird
 
+inline bool isNegativeInf(double x)
+{
+#ifdef WIN_NT
+	return _fpclass(x) == _FPCLASS_NINF;
+#else
+	return x == -INFINITY;
+#endif
+}
+} //namespace Firebird
 
 // getting a portable isinf() is harder than you would expect
 #ifdef WIN_NT
@@ -222,25 +233,17 @@ inline bool isinf(double x)
 {
 	return (!_finite (x) && !isnan(x));
 }
-#elif !defined(DARWIN)
+#else //WIN_NT
+#ifdef HAVE_CMATH
+using std::isinf;
+#else //HAVE_CMATH
 #ifndef isinf
 template <typename F>
 inline bool isinf(F x)
 {
 	return !isnan(x) && isnan(x - x);
 }
-#endif // isinf
-#endif // WIN_NT
-
-namespace Firebird {
-	inline bool isNegativeInf(double x)
-	{
-#ifdef WIN_NT
-		return _fpclass(x) == _FPCLASS_NINF;
-#else
-		return x == -INFINITY;
-#endif
-	}
-}
-
+#endif //isinf
+#endif //HAVE_CMATH
+#endif //WIN_NT
 #endif //CLASSES_FPE_CONTROL_H
