@@ -1600,6 +1600,13 @@ const ISC_STATUS* const origen = v;
 	return false;
 }
 
+inline bool sqlSymbolChar(char c, bool first)
+{
+	if (c & 0x80)
+		return false;
+	return (isdigit(c) && !first) || isalpha(c) || c == '_' || c == '$';
+}
+
 const char* dpbItemUpper(const char* s, FB_SIZE_T l, Firebird::string& buf)
 {
 	if (l && (s[0] == '"' || s[0] == '\''))
@@ -1612,30 +1619,37 @@ const char* dpbItemUpper(const char* s, FB_SIZE_T l, Firebird::string& buf)
 		{
 			if (s[i] == end_quote)
 			{
-				if (++i >= l || s[i] != end_quote)
-					break;		// delimited quote, done processing
+				if (++i >= l)
+				{
+					if (ascii && s[0] == '\'')
+						buf.upper();
+
+					return buf.c_str();
+				}
+
+				if (s[i] != end_quote)
+				{
+					buf.assign(&s[i], l - i);
+					Firebird::fatal_exception::raiseFmt("Invalid text <%s> after quoted string", buf.c_str());
+				}
 
 				// skipped the escape quote, continue processing
 			}
-
-			if (s[i] & 0x80)
+			else if (!sqlSymbolChar(s[i], i == 1))
 				ascii = false;
+
 			buf += s[i];
 		}
 
-		if (ascii && s[0] == '\'')
-			buf.upper();
-
-		return buf.c_str();
+		Firebird::fatal_exception::raiseFmt("Missing terminating quote <%c> in the end of quoted string", s[0]);
 	}
 
 	// non-quoted string - try to uppercase
 	for (FB_SIZE_T i = 0; i < l; ++i)
 	{
-		if (!(s[i] & 0x80))
-			buf += toupper(s[i]);
-		else
+		if (!sqlSymbolChar(s[i], i == 0))
 			return NULL;				// contains non-ascii data
+		buf += toupper(s[i]);
 	}
 
 	return buf.c_str();
