@@ -257,8 +257,9 @@ public:
 
 		if (!att)
 		{
-			dataFlag = true;
 			downFlag = isDown;
+			if (!isDown)
+				dataFlag = true;
 			return;
 		}
 
@@ -611,14 +612,17 @@ public:
 
 	void shutdown()
 	{
+		if (!sharedMemory)
+			return;
+		MutexLockGuard gLocal(initMutex, FB_FUNCTION);
 		if (!sharedMemory)
 			return;
 
 		{
 			Guard gShared(this);
-			MappingHeader* sMem = sharedMemory->getHeader();
 
-			startupSemaphore.tryEnter(5);
+			MappingHeader* sMem = sharedMemory->getHeader();
+			fb_assert(sMem->process[process].id);
 			sMem->process[process].flags &= ~MappingHeader::FLAG_ACTIVE;
 			(void)  // Ignore errors in cleanup
 				sharedMemory->eventPost(&sMem->process[process].notifyEvent);
@@ -628,17 +632,14 @@ public:
 			sharedMemory->eventFini(&sMem->process[process].notifyEvent);
 			sharedMemory->eventFini(&sMem->process[process].callbackEvent);
 
-			bool found = false;
-			for (unsigned n = 0; n < sMem->processes; ++n)
+			while (sMem->processes)
 			{
-				if (sMem->process[n].flags & MappingHeader::FLAG_ACTIVE)
-				{
-					found = true;
+				if (sMem->process[sMem->processes - 1].flags & MappingHeader::FLAG_ACTIVE)
 					break;
-				}
+				sMem->processes--;
 			}
 
-			if (!found)
+			if (!sMem->processes)
 				sharedMemory->removeMapFile();
 		}
 		sharedMemory = NULL;
@@ -700,15 +701,22 @@ public:
 			{
 				(Arg::Gds(isc_random) << "Error posting notifyEvent in mapping shared memory").raise();
 			}
+
+			int tout = 0;
 			while (sharedMemory->eventWait(&current->callbackEvent, value, 10000) != FB_SUCCESS)
 			{
 				if (!ISC_check_process_existence(p->id))
 				{
+					MAP_DEBUG(fprintf(stderr, "clearMapping: dead process found %d", p->id));
+
 					p->flags &= ~MappingHeader::FLAG_ACTIVE;
 					sharedMemory->eventFini(&p->notifyEvent);
 					sharedMemory->eventFini(&p->callbackEvent);
 					break;
 				}
+
+				if (++tout >= 1000) // 10 sec
+					(Arg::Gds(isc_random) << "Timeout when waiting callback from other process.").raise();
 			}
 			MAP_DEBUG(fprintf(stderr, "Notified pid %d about reset map %s\n", p->id, sMem->databaseForReset));
 		}
@@ -722,33 +730,54 @@ public:
 		if (sharedMemory)
 			return;
 
-		Arg::StatusVector statusVector;
+		AutoSharedMemory tempSharedMemory;
 		try
 		{
-			sharedMemory.reset(FB_NEW_POOL(*getDefaultMemoryPool())
+			tempSharedMemory.reset(FB_NEW_POOL(*getDefaultMemoryPool())
 				SharedMemory<MappingHeader>("fb_user_mapping", DEFAULT_SIZE, this));
 		}
 		catch (const Exception& ex)
 		{
+			StaticStatusVector s;
+			ex.stuffException(s);
 			iscLogException("MappingIpc: Cannot initialize the shared memory region", ex);
 			throw;
 		}
-		fb_assert(sharedMemory->getHeader()->mhb_header_version == MemoryHeader::HEADER_VERSION);
-		fb_assert(sharedMemory->getHeader()->mhb_version == MAPPING_VERSION);
+		fb_assert(tempSharedMemory->getHeader()->mhb_header_version == MemoryHeader::HEADER_VERSION);
+		fb_assert(tempSharedMemory->getHeader()->mhb_version == MAPPING_VERSION);
 
-		Guard gShared(this);
+		Guard gShared(tempSharedMemory);
 
-		MappingHeader* sMem = sharedMemory->getHeader();
+		MappingHeader* sMem = tempSharedMemory->getHeader();
 
-		for (process = 0; process < sMem->processes; ++process)
+		process = sMem->processes;
+		for (unsigned idx = 0; idx < sMem->processes; ++idx)
 		{
-			if (!(sMem->process[process].flags & MappingHeader::FLAG_ACTIVE))
-				break;
-			if (!ISC_check_process_existence(sMem->process[process].id))
+			MappingHeader::Process& p = sMem->process[idx];
+
+			if (p.id == processId)
 			{
-				sharedMemory->eventFini(&sMem->process[process].notifyEvent);
-				sharedMemory->eventFini(&sMem->process[process].callbackEvent);
-				break;
+				if (p.flags & MappingHeader::FLAG_ACTIVE) {
+					MAP_DEBUG(fprintf(stderr, "MappingIpc::setup: found existing entry for pid %d", processId));
+				}
+
+				process = idx;
+				continue;
+			}
+
+			if ((p.flags & MappingHeader::FLAG_ACTIVE) && !ISC_check_process_existence(p.id))
+			{
+				MAP_DEBUG(fprintf(stderr, "MappingIpc::setup: dead process found %d", p.id));
+
+				p.flags = 0;
+				tempSharedMemory->eventFini(&p.notifyEvent);
+				tempSharedMemory->eventFini(&p.callbackEvent);
+			}
+
+			if (!(p.flags & MappingHeader::FLAG_ACTIVE))
+			{
+				if (process == sMem->processes)
+					process = idx;
 			}
 		}
 
@@ -764,6 +793,8 @@ public:
 
 		sMem->process[process].id = processId;
 		sMem->process[process].flags = MappingHeader::FLAG_ACTIVE;
+		sharedMemory.reset(tempSharedMemory.release());
+
 		if (sharedMemory->eventInit(&sMem->process[process].notifyEvent) != FB_SUCCESS)
 		{
 			(Arg::Gds(isc_random) << "Error initializing notifyEvent in mapping shared memory").raise();
@@ -782,6 +813,7 @@ public:
 			sMem->process[process].flags &= ~MappingHeader::FLAG_ACTIVE;
 			throw;
 		}
+		startupSemaphore.enter();
 	}
 
 	void exceptionHandler(const Exception& ex, ThreadFinishSync<MappingIpc*>::ThreadRoutine*)
@@ -804,6 +836,7 @@ private:
 				if (p->flags & MappingHeader::FLAG_DELIVER)
 				{
 					resetMap(sharedMemory->getHeader()->databaseForReset);
+					p->flags &= ~MappingHeader::FLAG_DELIVER;
 
 					MappingHeader* sMem = sharedMemory->getHeader();
 					MappingHeader::Process* cur = &sMem->process[sMem->currentProcess];
@@ -811,7 +844,6 @@ private:
 					{
 						(Arg::Gds(isc_random) << "Error posting callbackEvent in mapping shared memory").raise();
 					}
-					p->flags &= ~MappingHeader::FLAG_DELIVER;
 				}
 
 				if (startup)
@@ -863,26 +895,36 @@ private:
 
 	class Guard;
 	friend class Guard;
+	typedef SharedMemory<MappingHeader> MappingSharedMemory;
+	typedef AutoPtr<MappingSharedMemory> AutoSharedMemory;
 
 	class Guard
 	{
 	public:
 		explicit Guard(MappingIpc* ptr)
+			: data(ptr->sharedMemory)
+		{
+			fb_assert(data);
+			data->mutexLock();
+		}
+
+		explicit Guard(MappingSharedMemory* ptr)
 			: data(ptr)
 		{
-			data->sharedMemory->mutexLock();
+			fb_assert(data);
+			data->mutexLock();
 		}
 
 		~Guard()
 		{
-			data->sharedMemory->mutexUnlock();
+			data->mutexUnlock();
 		}
 
 	private:
 		Guard(const Guard&);
 		Guard& operator=(const Guard&);
 
-		MappingIpc* const data;
+		MappingSharedMemory* const data;
 	};
 
 	static void clearDelivery(MappingIpc* mapping)
@@ -890,7 +932,7 @@ private:
 		mapping->clearDeliveryThread();
 	}
 
-	AutoPtr<SharedMemory<MappingHeader> > sharedMemory;
+	AutoSharedMemory sharedMemory;
 	Mutex initMutex;
 	const SLONG processId;
 	unsigned process;
@@ -1034,10 +1076,10 @@ bool mapUser(string& name, string& trusted_role, Firebird::string* auth_method,
 			if (syncType == SYNC_EXCLUSIVE)
 			{
 				if (!iSec)
-					iSec.attach(st, securityAlias, cryptCb);
+					secDown = iSec.attach(st, securityAlias, cryptCb);
 
 				if (db && !iDb)
-					iDb.attach(st, alias, cryptCb);
+					dbDown = iDb.attach(st, alias, cryptCb);
 			}
 
 			MutexEnsureUnlock g(treeMutex, FB_FUNCTION);
