@@ -195,7 +195,6 @@ void StatusXcp::as_sqlstate(char* sqlstate) const
 static void execute_looper(thread_db*, jrd_req*, jrd_tra*, const StmtNode*, jrd_req::req_s);
 static void looper_seh(thread_db*, jrd_req*, const StmtNode*);
 static void release_blobs(thread_db*, jrd_req*);
-static void release_proc_save_points(jrd_req*);
 static void trigger_failure(thread_db*, jrd_req*);
 static void stuff_stack_trace(const jrd_req*);
 
@@ -610,7 +609,7 @@ void EXE_receive(thread_db* tdbb,
 	DEV_BLKCHK(request, type_req);
 
 	if (--tdbb->tdbb_quantum < 0)
-		JRD_reschedule(tdbb, 0, true);
+		JRD_reschedule(tdbb, true);
 
 	jrd_tra* transaction = request->req_transaction;
 
@@ -700,7 +699,7 @@ void EXE_receive(thread_db* tdbb,
 			Savepoint* const save_sav_point = transaction->tra_save_point;
 			transaction->tra_save_point = request->req_proc_sav_point;
 			request->req_proc_sav_point = save_sav_point;
-			release_proc_save_points(request);
+			EXE_release_proc_save_points(request);
 		}
 		throw;
 	}
@@ -757,7 +756,7 @@ void EXE_send(thread_db* tdbb, jrd_req* request, USHORT msg, ULONG length, const
 	DEV_BLKCHK(request, type_req);
 
 	if (--tdbb->tdbb_quantum < 0)
-		JRD_reschedule(tdbb, 0, true);
+		JRD_reschedule(tdbb, true);
 
 	if (!(request->req_flags & req_active))
 		ERR_post(Arg::Gds(isc_req_sync));
@@ -939,7 +938,7 @@ void EXE_unwind(thread_db* tdbb, jrd_req* request)
 	request->req_sorts.unlinkAll();
 
 	if (request->req_proc_sav_point && (request->req_flags & req_proc_fetch))
-		release_proc_save_points(request);
+		EXE_release_proc_save_points(request);
 
 	TRA_detach_request(request);
 
@@ -1228,7 +1227,8 @@ const StmtNode* EXE_looper(thread_db* tdbb, jrd_req* request, const StmtNode* no
 	jrd_tra* sysTransaction = attachment->getSysTransaction();
 	Database* dbb = tdbb->getDatabase();
 
-	if (!node || node->kind != DmlNode::KIND_STATEMENT)
+	// ASF: It's already a StmtNode, so do not do a virtual call in execution.
+	if (!node)	/// if (!node || node->getKind() != DmlNode::KIND_STATEMENT
 		BUGCHECK(147);
 
 	// Save the old pool and request to restore on exit
@@ -1254,7 +1254,7 @@ const StmtNode* EXE_looper(thread_db* tdbb, jrd_req* request, const StmtNode* no
 			if (request->req_operation == jrd_req::req_evaluate)
 			{
 				if (--tdbb->tdbb_quantum < 0)
-					JRD_reschedule(tdbb, 0, true);
+					JRD_reschedule(tdbb, true);
 
 				if (node->hasLineColumn)
 				{
@@ -1480,18 +1480,18 @@ static void release_blobs(thread_db* tdbb, jrd_req* request)
 }
 
 
-static void release_proc_save_points(jrd_req* request)
+void EXE_release_proc_save_points(jrd_req* request)
 {
-/**************************************
+/************************************************************
  *
- *	r e l e a s e _ p r o c _ s a v e _ p o i n t s
+ *	E X E _ r e l e a s e _ p r o c _ s a v e _ p o i n t s
  *
- **************************************
+ ************************************************************
  *
  * Functional description
  *	Release savepoints used by this request.
  *
- **************************************/
+ ************************************************************/
 	Savepoint* sav_point = request->req_proc_sav_point;
 
 	if (request->req_transaction)
