@@ -31,6 +31,8 @@
 
 #include "../common/os/path_utils.h"
 #include "../jrd/err_proto.h"
+#include "../common/os/fbsyslog.h"
+#include "../common/StatusArg.h"
 #include "../common/isc_proto.h"
 #include "../common/classes/fb_string.h"
 #include "../common/classes/init.h"
@@ -42,6 +44,7 @@
 #include "../common/classes/GenericMap.h"
 #include "../common/db_alias.h"
 #include "../common/dllinst.h"
+#include "../common/file_params.h"
 
 #include "../yvalve/config/os/config_root.h"
 
@@ -270,6 +273,7 @@ namespace
 	{
 		LocalStatus ls;
 		CheckStatusWrapper s(&ls);
+
 		if (pluginLoaderConfig)
 		{
 			const ConfigFile::Parameter* p = pluginLoaderConfig->findParameter("Config");
@@ -292,19 +296,32 @@ namespace
 
 
 	// Plugins registered when loading plugin module.
-	// This is POD object - no dtor, only simple data types inside.
 	struct RegisteredPlugin
 	{
 		RegisteredPlugin(IPluginFactory* f, const char* nm, unsigned int t)
 			: factory(f), name(nm), type(t)
 		{ }
 
+		RegisteredPlugin(MemoryPool& p, IPluginFactory* f, const char* nm, unsigned int t)
+			: factory(f), name(p), type(t)
+		{
+			name = nm;
+		}
+
 		RegisteredPlugin()
-			: factory(NULL), name(NULL), type(0)
+			: factory(NULL), name(), type(0)
+		{ }
+
+		RegisteredPlugin(MemoryPool& p)
+			: factory(NULL), name(p), type(0)
+		{ }
+
+		RegisteredPlugin(MemoryPool& p, const RegisteredPlugin& from)
+			: factory(from.factory), name(p, from.name), type(from.type)
 		{ }
 
 		IPluginFactory* factory;
-		const char* name;
+		PathName name;
 		unsigned int type;
 	};
 
@@ -415,7 +432,7 @@ namespace
 		PathName name;
 		Firebird::AutoPtr<ModuleLoader::Module> module;
 		Firebird::IPluginModule* cleanup;
-		HalfStaticArray<RegisteredPlugin, 2> regPlugins;
+		ObjectsArray<RegisteredPlugin> regPlugins;
 		PluginModule* next;
 		PluginModule** prev;
 	};
@@ -839,8 +856,8 @@ namespace
 		{
 			namesList.assign(pnamesList);
 			namesList.alltrim(" \t");
-			Firebird::LocalStatus s;
-			Firebird::CheckStatusWrapper statusWrapper(&s);
+			LocalStatus ls;
+			CheckStatusWrapper statusWrapper(&ls);
 			next(&statusWrapper);
 			check(&statusWrapper);
 		}
@@ -978,6 +995,9 @@ namespace
 			current = rc;
 			startModule(masterInterface);
 			current = NULL;
+#ifdef DARWIN	// Plugin unload disabled in MacOS - GH-7112
+			rc->addRef();
+#endif
 			return rc;
 		}
 
@@ -1041,26 +1061,56 @@ PluginManager::PluginManager()
 
 void PluginManager::registerPluginFactory(unsigned int interfaceType, const char* defaultName, IPluginFactory* factory)
 {
-	MutexLockGuard g(plugins->mutex, FB_FUNCTION);
-
-	if (!current)
+	try
 	{
-		// not good time to call this function - ignore request
-		gds__log("Unexpected call to register plugin %s, type %d - ignored\n", defaultName, interfaceType);
-		return;
+		MutexLockGuard g(plugins->mutex, FB_FUNCTION);
+
+		if (!current)
+		{
+			// not good time to call this function - ignore request
+			gds__log("Unexpected call to register plugin %s, type %d - ignored\n", defaultName, interfaceType);
+			return;
+		}
+
+		unsigned int r = current->addPlugin(RegisteredPlugin(factory, defaultName, interfaceType));
+
+		if (current == builtin)
+		{
+			PathName plugConfigFile = fb_utils::getPrefix(IConfigManager::DIR_PLUGINS, defaultName);
+			changeExtension(plugConfigFile, "conf");
+
+			ConfiguredPlugin* p = FB_NEW ConfiguredPlugin(RefPtr<PluginModule>(builtin), r,
+				findInPluginsConf("Plugin", defaultName), plugConfigFile, defaultName);
+			p->addRef();  // Will never be unloaded
+			plugins->put(MapKey(interfaceType, defaultName), p);
+		}
 	}
-
-	unsigned int r = current->addPlugin(RegisteredPlugin(factory, defaultName, interfaceType));
-
-	if (current == builtin)
+	catch(const Exception& ex)
 	{
-		PathName plugConfigFile = fb_utils::getPrefix(IConfigManager::DIR_PLUGINS, defaultName);
-		changeExtension(plugConfigFile, "conf");
+		// looks like something gone seriously wrong - therefore add more error handling here
+		try
+		{
+			LocalStatus ls;
+			CheckStatusWrapper s(&ls);
+			ex.stuffException(&s);
+			char text[256];
+			UtilInterfacePtr()->formatStatus(text, sizeof(text), &s);
+			Syslog::Record(Syslog::Error, text);
 
-		ConfiguredPlugin* p = FB_NEW ConfiguredPlugin(RefPtr<PluginModule>(builtin), r,
-									findInPluginsConf("Plugin", defaultName), plugConfigFile, defaultName);
-		p->addRef();  // Will never be unloaded
-		plugins->put(MapKey(interfaceType, defaultName), p);
+			iscLogException("Plugin registration error", ex);
+		}
+		catch(const BadAlloc&)
+		{
+			Syslog::Record(Syslog::Error, "Plugin registration error - out of memory");
+		}
+		catch(...)
+		{
+			Syslog::Record(Syslog::Error, "Double fault during plugin registration");
+		}
+
+#ifdef DEV_BUILD
+		abort();
+#endif
 	}
 }
 
