@@ -457,29 +457,23 @@ void UtilInterface::getFbVersion(CheckStatusWrapper* status, IAttachment* att,
 			if (status->getState() & Firebird::IStatus::STATE_ERRORS)
 				return;
 
-			const UCHAR* p = buf;
-			redo = false;
+			ClumpletReader p(ClumpletReader::InfoResponse, buf, buf_len);
 
-			while (!redo && *p != isc_info_end && p < buf + buf_len)
+			for (redo = false; !(redo || p.isEof()); p.moveNext())
 			{
-				const UCHAR item = *p++;
-				const USHORT len = static_cast<USHORT>(gds__vax_integer(p, 2));
-
-				p += 2;
-
-				switch (item)
+				switch (p.getClumpTag())
 				{
 				case isc_info_firebird_version:
-					versions = (TEXT*) p;
+					versions = (TEXT*) p.getBytes();
 					break;
 
 				case isc_info_implementation:
-					implementations = (TEXT*) p;
+					implementations = (TEXT*) p.getBytes();
 					break;
 
 				case fb_info_implementation:
-					dbis = p;
-					if (dbis[0] * 6 + 1 > len)
+					dbis = p.getBytes();
+					if (dbis[0] * 6u + 1u > p.getClumpLength())
 					{
 						// fb_info_implementation value appears incorrect
 						dbis = NULL;
@@ -492,13 +486,13 @@ void UtilInterface::getFbVersion(CheckStatusWrapper* status, IAttachment* att,
 
 				case isc_info_truncated:
 					redo = true;
+					// fall down...
+				case isc_info_end:
 					break;
 
 				default:
 					(Arg::Gds(isc_random) << "Invalid info item").raise();
 				}
-
-				p += len;
 			}
 
 			// Our buffer wasn't large enough to hold all the information,
@@ -1121,37 +1115,35 @@ int API_ROUTINE gds__blob_size(FB_API_HANDLE* b, SLONG* size, SLONG* seg_count,
  *
  **************************************/
 	ISC_STATUS_ARRAY status_vector;
-	SCHAR buffer[64];
+	UCHAR buffer[64];
 
-	if (isc_blob_info(status_vector, b, sizeof(blob_items), blob_items, sizeof(buffer), buffer))
+	if (isc_blob_info(status_vector, b, sizeof(blob_items), blob_items, sizeof(buffer), (SCHAR*)buffer))
 	{
 		isc_print_status(status_vector);
 		return FALSE;
 	}
 
-	const UCHAR* p = reinterpret_cast<UCHAR*>(buffer);
-	UCHAR item;
-	while ((item = *p++) != isc_info_end)
+	for (ClumpletReader p(ClumpletReader::InfoResponse, buffer, sizeof(buffer)); !p.isEof(); p.moveNext())
 	{
-		const USHORT l = gds__vax_integer(p, 2);
-		p += 2;
-		const SLONG n = gds__vax_integer(p, l);
-		p += l;
+		UCHAR item = p.getClumpTag();
+		if (item == isc_info_end)
+			break;
+
 		switch (item)
 		{
 		case isc_info_blob_max_segment:
 			if (max_seg)
-				*max_seg = n;
+				*max_seg = p.getInt();
 			break;
 
 		case isc_info_blob_num_segments:
 			if (seg_count)
-				*seg_count = n;
+				*seg_count = p.getInt();
 			break;
 
 		case isc_info_blob_total_length:
 			if (size)
-				*size = n;
+				*size = p.getInt();
 			break;
 
 		default:
@@ -2663,16 +2655,10 @@ static void get_ods_version(CheckStatusWrapper* status, IAttachment* att,
 	if (status->getState() & Firebird::IStatus::STATE_ERRORS)
 		return;
 
-	const UCHAR* p = buffer;
-	UCHAR item;
-
-	while ((item = *p++) != isc_info_end)
+	for (ClumpletReader p(ClumpletReader::InfoResponse, buffer, sizeof(buffer)); !p.isEof(); p.moveNext())
 	{
-		const USHORT l = static_cast<USHORT>(gds__vax_integer(p, 2));
-		p += 2;
-		const USHORT n = static_cast<USHORT>(gds__vax_integer(p, l));
-		p += l;
-		switch (item)
+		const USHORT n = static_cast<USHORT>(p.getInt());
+		switch (p.getClumpTag())
 		{
 		case isc_info_ods_version:
 			*ods_version = n;
@@ -2682,6 +2668,9 @@ static void get_ods_version(CheckStatusWrapper* status, IAttachment* att,
 			*ods_minor_version = n;
 			break;
 
+		case isc_info_end:
+			break;
+
 		default:
 			(Arg::Gds(isc_random) << "Invalid info item").raise();
 		}
@@ -2934,7 +2923,7 @@ void makeKey()
 	int err = pthread_key_create(&key, ThreadCleanup::destructor);
 	if (err)
 	{
-		Firebird::system_call_failed("pthread_key_create", err);
+		Firebird::system_call_failed::raise("pthread_key_create", err);
 	}
 	keySet = true;
 }
@@ -2944,13 +2933,13 @@ void ThreadCleanup::initThreadCleanup()
 	int err = pthread_once(&keyOnce, makeKey);
 	if (err)
 	{
-		Firebird::system_call_failed("pthread_once", err);
+		Firebird::system_call_failed::raise("pthread_once", err);
 	}
 
 	err = pthread_setspecific(key, &key);
 	if (err)
 	{
-		Firebird::system_call_failed("pthread_setspecific", err);
+		Firebird::system_call_failed::raise("pthread_setspecific", err);
 	}
 }
 
@@ -2974,7 +2963,7 @@ public:
 		{
 			int err = pthread_key_delete(key);
 			if (err)
-				Firebird::system_call_failed("pthread_key_delete", err);
+				gds__log("pthread_key_delete failed with error %d", err);
 		}
 	}
 };
